function [tstar, Sample, r0c, sample, ni] = clusterkd(k, nu, n)

% Runs the k-dimensional clustering example of Section 5.2 with k=3
% To duplicate what is there:
% [tstar, Sample, r0c, sample, ni] = clusterkd(3, .5, 80);

close all
rng(111); % to set the random seed

c = nu^k;
b = 1/c;
r0 = min(1,sqrt(k)/2);
r0c = r0*(c^(1/k));

% Set t*
corners = (-1).^ff2n(k);
axials = [sqrt(k)*eye(k); -sqrt(k)*eye(k)];
tstar = [corners; axials; zeros(1,k)]';
q = size(tstar,2);
which = q; % sphere to be analyzed

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Draw a sample of size n
% The sample from S(ti,R) with be the ni(i) columns of  Sample{i}

weights = [ones(q-1,1);.5];
weights = weights/sum(weights);
ni = floor(n*weights);
ni(q) = n- sum(ni(1:(q-1)));

Theta = zeros(k-1,n);
rho = betarnd(k, b, 1, n);
for j = 1:(k-2)
    Theta(j,:) = acos(sqrt(betarnd(.5, .5*(k-j), 1, n)));
end
Theta(k-1,:) = 2*pi*rand(1,n) - pi;
Theta = ((-1).^binornd(1,.5,size(Theta))).*Theta;
Stheta = sin(Theta);
Ctheta = cos(Theta);
Y = zeros(k,n);
Y(1,:) = Stheta(1,:);
for j = 2:(k-1)
    Y(j,:) = prod(Ctheta(1:(j-1),:),1).*Stheta(j,:);
end
Y(k,:) = prod(Ctheta(1:(k-1),:),1);
Y = rho.*Y;
Sample{1} = tstar(:,1) + r0c*Y(:,1:ni(1));
for j = 2:q
    Sample{j} = tstar(:,j) + r0c*Y(:,(sum(ni(1:(j-1)))+1):sum(ni(1:j)));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if k==3
    newscale=.9;
    figure(1) 
    subplot(2,2,1)
    hold on
    fontsize(scale=newscale)
    scatter3(tstar(1,:),tstar(2,:),tstar(3,:))
    xlabel('x1')
    ylabel('x2')
    zlabel('x3')
    [x,y,z] = sphere; % unit sphere S(0,1)
    x = x*r0c;
    y = y*r0c;
    z = z*r0c;
    for j = 1:q
        %surf(x+tstar(1,j),y+tstar(2,j),z+tstar(3,j),'EdgeColor','yellow');
        scatter3(tstar(1,j),tstar(2,j),tstar(3,j), 'Marker', 'o', 'MarkerFaceColor', 'k')
        mesh(x+tstar(1,j),y+tstar(2,j),z+tstar(3,j),'FaceAlpha', '0');
    end
    view(3)
    hold off

    subplot(2,2,2)
    hold on
    fontsize(scale=newscale)
    [x,y,z] = sphere; % unit sphere S(0,1)
    j1 = which;  
    x = x*r0c+tstar(1,j1);
    y = y*r0c+tstar(2,j1);
    z = z*r0c+tstar(3,j1);
    scatter3(tstar(1,j1),tstar(2,j1),tstar(3,j1), 'Marker', 'o', 'MarkerFaceColor', 'k')
    sample = Sample{j1};
    mesh(x,y,z,'FaceAlpha', '0')
    scatter3(sample(1,:),sample(2,:),sample(3,:), 'Marker', '*', 'MarkerFaceColor', 'r');
    xlabel('x1');
    ylabel('x2');
    zlabel('x3');
    view(3)
    hold off

    subplot(2,2,3)
    hold on
    fontsize(scale=newscale)
    title('(a)', 'FontWeight', 'normal')
    axis off
    hold off
    subplot(2,2,4)
    hold on
    fontsize(scale=newscale)
    axis off
    title('(b)', 'FontWeight', 'normal')
end

