function [Table1, misc] = jitter(c, nu, n, quant, Nsamples)

% Carries out the calculations for Section 3 of the paper. 
% Enter this command to duplicate that example:
% [Table1, misc] = jitter(.5, .5, 10, .9, 1000);


%close all
%numtoplot = 100;
M = 1000;
rng(192); % to set the random seed
newscale=1;

%Compute alpha, given nu
func = @(a)  1/(1 + (9*(3 - 5*a)^2)/(25*(1 - 3*a)^3)) - nu;
alpha = fsolve(func, -1);

% Compute t*
i = (1:n)';
s = -(1-3*alpha)*(2*i-1-n)/n;
Delta = (s.^2)/4 - alpha^3;
tstar = nthroot((-s/2 + sqrt(Delta)),3) + nthroot((-s/2 - sqrt(Delta)),3);
c = min([c,n*(1+tstar(1))]);

% Compute max losses at one 'c', and other parameters of m(x)
mu2xi = .2*(3-5*alpha)/(1-3*alpha);
kappa0xi = .3*(3 - 10*alpha + 15*alpha^2)/((1-3*alpha)^2);
kappa2xi = 3*(15 - 42*alpha + 35*alpha^2)/(70*(1-3*alpha)^2);
maxlossXI = 2*(1-nu)*(1+1/(3*mu2xi)) + nu*(1+1.25*(3*mu2xi - 1)^2);
lambdastar = sum(tstar.^2)/n;
lambda2 = lambdastar + (c^2)/(3*n^2);
maxlossPHI = 2*(1-nu)*(1+1/(3*lambda2)) + nu*max(1,1/(3*lambda2))/c;

% Compute max losses for PHI at ALL 'c'
C = linspace(.02, min(1,n*(1 + tstar(1))));
C = C';
Lambda2 = lambdastar + C.^2/(3*(n^2));
MAXlossPHI = 2*(1-nu)*(1+1./(3*Lambda2)) + nu*max(1, 1./(3*Lambda2))./C;

%Plot:
figure(1)
% Plot m
subplot(1,3,1)
hold on
fontsize(scale = newscale)
phisupport = [tstar - c/n, tstar + c/n];
line([phisupport(:,1),phisupport(:,1)], [0,1/(2*c)], 'Color', 'black')
line([phisupport(:,2),phisupport(:,2)], [0,1/(2*c)], 'Color', 'black')
line([phisupport(:,1),phisupport(:,2)], [1/(2*c),1/(2*c)], 'Color', 'black')
plot(tstar,0, 'r.');
xlabel('(a)');
hold off

% Plot max losses
subplot(1,3,3)
hold on
fontsize(scale = newscale)
xlim([0,min(1,n*(1 + tstar(1)))])
ylim([0,max(MAXlossPHI)]);
plot(C,MAXlossPHI);
mylabel = {['c = ', num2str(c),',']; ['I_{\nu}(\Phi) = ', num2str(round(maxlossPHI,2))]};
plot(c,maxlossPHI,'r*')
text(c,maxlossPHI,mylabel,'VerticalAlignment','bottom','HorizontalAlignment','right')
%xlabel({'(c): max IMSE I_{\nu}(\Phi_n) vs. c;', ...
%    ['Horizontal line at I_{\nu}(\xi) = ', num2str(round(maxlossXI,3)), '.']});
xlabel('(c)');
line([0,min(1,n*(1 + tstar(1)))],[maxlossXI,maxlossXI], 'Color', 'black');
hold off

% Plot least favourable psi for Phi
subplot(1,3,2)
hold on
fontsize(scale = newscale)
x = linspace(-1,1,201)';
%ylim([-2*c,2*c])
IndS = [];
for j = 1:length(x)
    IndS(j) = sum((phisupport(:,1) <= x(j)).*(phisupport(:,2) >= x(j)));
end
IndS = IndS';
gamma0 = max(1, 1/(3*lambda2))/c;
psiPhi = (IndS - 1/gamma0)/sqrt(2*c*(1 - 1/gamma0)).*((x/lambda2).^(lambda2<1/3));
sz = 3;
scatter(x, psiPhi, sz);
xlabel('(b)');
hold off


%generate many samples:
SAMPLE2 = zeros(n,M);
sampleloss1 = zeros(n,1);
sampleloss2 = sampleloss1;
sampleimse_1 = sampleloss1;
sampleimse_2 = sampleloss1;

for j = 1:M
    SAMPLE2(:,j) = unifrnd(phisupport(:,1), phisupport(:,2)); % stratified - random only within bins
end

SAMPLE1 = reshape(SAMPLE2, [n*M,1]);
p = randperm(n*M);
SAMPLE1 = SAMPLE1(p);
SAMPLE1 = reshape(SAMPLE1, [n,M]);  % truly random
isb = max(1,1/(3*lambda2))/c;
for j = 1:M
    x1 = SAMPLE1(:,j);  % random within bins; stratified
    x2 = SAMPLE2(:,j);  % random from phi
    samplemu2_1 = sum(x1.^2)/n;
    samplemu2_2 = sum(x2.^2)/n;
    sampleloss1(j) = 2*(1-nu)*(1+1/(3*samplemu2_1)) + nu*(1.25*(3*samplemu2_1-1)^2+1);
    sampleloss2(j) = 2*(1-nu)*(1+1/(3*samplemu2_2)) + nu*(1.25*(3*samplemu2_2-1)^2+1);

    samplemean_1 = mean(x1);
    samplemean_2 = mean(x2);
    samplevar_1 = var(x1)*(n-1)/n;
    samplevar_2 = var(x2)*(n-1)/n;
    sampleIV_1 = 2*(1 + ((samplemean_1^2 + 1/3)/samplevar_1));
    sampleIV_2 = 2*(1 + ((samplemean_2^2 + 1/3)/samplevar_2));
    sampleimse_1(j) = (1-nu)*sampleIV_1 + nu*isb;
    sampleimse_2(j) = (1-nu)*sampleIV_2 + nu*isb;
end

% Compute the values appearing in Table 1
%quadlossstar = 2*(1-nu)*(1+1/(3*lambdastar)) + nu*(1.25*(3*lambdastar-1)^2+1); % = QIMSE(c=0). Depends only on tstar.

quantloss1 = quantile(sampleloss1, quant);
meanloss1 = mean(sampleloss1);
quantloss2 = quantile(sampleloss2, quant);
meanloss2 = mean(sampleloss2);

Table1 = [quantloss1, quantloss2; meanloss1, meanloss2];
newscale = .5;

figure(2) % Plot the histograms of sample losses
% Plot the histograms of IMSE's
% Compare with meanimse and maxlossPHI
subplot(2,2,3)
sampleimse = sampleimse_1;
meanimse1 = mean(sampleimse);

hold on
%xlabel('(a): Completely random sampling');
xlabel('(c)');
histogram(sampleimse, 'HandleVisibility', 'off', 'FaceColor', 'yellow' )
text1 = (['Average j_{\nu}(\delta) = ', num2str(round(meanimse1,2))]);
plot(meanimse1, 0,  'k*', 'DisplayName', text1);
text2 = (['I_{\nu}(\Phi) = ', num2str(round(maxlossPHI,2))]);
plot(maxlossPHI, 0,  'g*', 'DisplayName', text2);
%plot(maxlossPHI, 0,  'g*')
legend ('Location', 'best')
hold off


subplot(2,2,4)
% Compare with meanimse and maxlossPHI
sampleimse = sampleimse_2;
meanimse2 = mean(sampleimse);

hold on
%xlabel('(a): Completely random sampling');
xlabel('(d)');
histogram(sampleimse, 'HandleVisibility', 'off', 'FaceColor', 'yellow' )
text1 = (['Average j_{\nu}(\delta) = ', num2str(round(meanimse2,2))]);
plot(meanimse2, 0,  'k*', 'DisplayName', text1);
text2 = (['I_{\nu}(\Phi) = ', num2str(round(maxlossPHI,2))]);
plot(maxlossPHI, 0,  'g*', 'DisplayName', text2);
%plot(maxlossPHI, 0,  'g*')
legend ('Location', 'best')
hold off

% Estimate IMSE

unifSamples = rand(n, Nsamples);
Samples = zeros(size(unifSamples));

for j = 1:Nsamples
    u = unifSamples(:,j);
    s =  -(1-3*alpha)*(2*u - 1);
    dd = (s.^2)/4 - alpha^3;
    Samples(:,j) = nthroot(-s/2 + sqrt(dd),3) + nthroot(-s/2 - sqrt(dd),3);
end


% Compute beta
A = diag([2, 2/3]);
Hxi = .5*diag([1, 3*mu2xi^2]);
Gxi = diag([(kappa0xi - .5), (kappa2xi - 1.5*mu2xi^2)]);
Mxi = diag([1, mu2xi]);
rootG = sqrt(Gxi);
P = eye(2) + rootG'*(Hxi\rootG);
[V,D] = eig(P);
[~,ind] = sort(diag(D));
%Ds = D(ind,ind);
Vs = V(:,ind);
beta = Vs(:,2);
betastar = rootG\beta; % G^(-1/2) times beta

% Estimate J(XI)
Jxi = zeros(Nsamples,1);
for j = 1:Nsamples
    x = Samples(:,j);
    F = [ones(size(x)), x];
    Dm = diag(3*(x.^2 - alpha)/(2*(1 - 3*alpha)));
    Mdelta = F'*F/n;
    Mm = F'*Dm*F/n;
    rightvec = (Mdelta\Mm - A\Mxi)*betastar;
    Jxi(j) = (1-nu)*trace(Mdelta\A) + nu*(1+rightvec'*A*rightvec);
end
JXI = mean(Jxi);

figure(3)
hold on
histogram(Jxi, 'HandleVisibility', 'off', 'FaceColor', 'yellow' )
text1 = (['Average imse = ', num2str(round(JXI,2))]);
plot(JXI, 0,  'k*', 'DisplayName', text1);
text2 = (['I_{\nu}(\xi) = ', num2str(round(maxlossXI,2))]);
plot(maxlossXI, 0,  'g*', 'DisplayName', text2);
legend ('Location', 'northeast')
hold off



miscellany = [c; lambda2; maxlossXI; maxlossPHI; meanimse1; meanimse2];
names = {'c'; 'lambda2'; 'maxlossXI'; 'maxlossPHI'; 'meanimse1'; 'meanimse2'};
misc = table(miscellany,'RowNames',names);
